/**
 * Copyright (c) 2017-2018 INCHRON GmbH
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     INCHRON GmbH - Meta Model design and implementation
 */
package com.inchron.realtime.root.model.fmi.variable.util;

import com.inchron.realtime.root.Referrable;
import com.inchron.realtime.root.model.LoadResource;
import com.inchron.realtime.root.model.ModelObject;
import com.inchron.realtime.root.model.TraceObject;

import com.inchron.realtime.root.model.fmi.IntegerAttributes;
import com.inchron.realtime.root.model.fmi.RealAttributes;

import com.inchron.realtime.root.model.fmi.variable.Enumeration;
import com.inchron.realtime.root.model.fmi.variable.Real;
import com.inchron.realtime.root.model.fmi.variable.ScalarVariable;
import com.inchron.realtime.root.model.fmi.variable.VariablePackage;

import com.inchron.realtime.root.model.peripheral.AbstractPort;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;

import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;

import org.eclipse.emf.ecore.EObject;

/**
 * <!-- begin-user-doc -->
 * The <b>Adapter Factory</b> for the model.
 * It provides an adapter <code>createXXX</code> method for each class of the model.
 * <!-- end-user-doc -->
 * @see com.inchron.realtime.root.model.fmi.variable.VariablePackage
 * @generated
 */
public class VariableAdapterFactory extends AdapterFactoryImpl {
	/**
	 * The cached model package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static VariablePackage modelPackage;

	/**
	 * Creates an instance of the adapter factory.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public VariableAdapterFactory() {
		if (modelPackage == null) {
			modelPackage = VariablePackage.eINSTANCE;
		}
	}

	/**
	 * Returns whether this factory is applicable for the type of the object.
	 * <!-- begin-user-doc -->
	 * This implementation returns <code>true</code> if the object is either the model's package or is an instance object of the model.
	 * <!-- end-user-doc -->
	 * @return whether this factory is applicable for the type of the object.
	 * @generated
	 */
	@Override
	public boolean isFactoryForType(Object object) {
		if (object == modelPackage) {
			return true;
		}
		if (object instanceof EObject) {
			return ((EObject)object).eClass().getEPackage() == modelPackage;
		}
		return false;
	}

	/**
	 * The switch that delegates to the <code>createXXX</code> methods.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected VariableSwitch<Adapter> modelSwitch =
		new VariableSwitch<Adapter>() {
			@Override
			public Adapter caseBoolean(com.inchron.realtime.root.model.fmi.variable.Boolean object) {
				return createBooleanAdapter();
			}
			@Override
			public Adapter caseEnumeration(Enumeration object) {
				return createEnumerationAdapter();
			}
			@Override
			public Adapter caseInteger(com.inchron.realtime.root.model.fmi.variable.Integer object) {
				return createIntegerAdapter();
			}
			@Override
			public Adapter caseReal(Real object) {
				return createRealAdapter();
			}
			@Override
			public Adapter caseScalarVariable(ScalarVariable object) {
				return createScalarVariableAdapter();
			}
			@Override
			public Adapter caseReferrable(Referrable object) {
				return createReferrableAdapter();
			}
			@Override
			public Adapter caseModelObject(ModelObject object) {
				return createModelObjectAdapter();
			}
			@Override
			public Adapter caseTraceObject(TraceObject object) {
				return createTraceObjectAdapter();
			}
			@Override
			public Adapter caseLoadResource(LoadResource object) {
				return createLoadResourceAdapter();
			}
			@Override
			public Adapter caseAbstractPort(AbstractPort object) {
				return createAbstractPortAdapter();
			}
			@Override
			public Adapter caseIntegerAttributes(IntegerAttributes object) {
				return createIntegerAttributesAdapter();
			}
			@Override
			public Adapter caseRealAttributes(RealAttributes object) {
				return createRealAttributesAdapter();
			}
			@Override
			public Adapter defaultCase(EObject object) {
				return createEObjectAdapter();
			}
		};

	/**
	 * Creates an adapter for the <code>target</code>.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param target the object to adapt.
	 * @return the adapter for the <code>target</code>.
	 * @generated
	 */
	@Override
	public Adapter createAdapter(Notifier target) {
		return modelSwitch.doSwitch((EObject)target);
	}


	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.variable.Boolean <em>Boolean</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.variable.Boolean
	 * @generated
	 */
	public Adapter createBooleanAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.variable.Enumeration <em>Enumeration</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.variable.Enumeration
	 * @generated
	 */
	public Adapter createEnumerationAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.variable.Integer <em>Integer</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.variable.Integer
	 * @generated
	 */
	public Adapter createIntegerAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.variable.Real <em>Real</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.variable.Real
	 * @generated
	 */
	public Adapter createRealAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.variable.ScalarVariable <em>Scalar Variable</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.variable.ScalarVariable
	 * @generated
	 */
	public Adapter createScalarVariableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.Referrable <em>Referrable</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.Referrable
	 * @generated
	 */
	public Adapter createReferrableAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.ModelObject <em>Object</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.ModelObject
	 * @generated
	 */
	public Adapter createModelObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.TraceObject <em>Trace Object</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.TraceObject
	 * @generated
	 */
	public Adapter createTraceObjectAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.LoadResource <em>Load Resource</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.LoadResource
	 * @generated
	 */
	public Adapter createLoadResourceAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.peripheral.AbstractPort <em>Abstract Port</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.peripheral.AbstractPort
	 * @generated
	 */
	public Adapter createAbstractPortAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.IntegerAttributes <em>Integer Attributes</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.IntegerAttributes
	 * @generated
	 */
	public Adapter createIntegerAttributesAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for an object of class '{@link com.inchron.realtime.root.model.fmi.RealAttributes <em>Real Attributes</em>}'.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null so that we can easily ignore cases;
	 * it's useful to ignore a case when inheritance will catch all the cases anyway.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @see com.inchron.realtime.root.model.fmi.RealAttributes
	 * @generated
	 */
	public Adapter createRealAttributesAdapter() {
		return null;
	}

	/**
	 * Creates a new adapter for the default case.
	 * <!-- begin-user-doc -->
	 * This default implementation returns null.
	 * <!-- end-user-doc -->
	 * @return the new adapter.
	 * @generated
	 */
	public Adapter createEObjectAdapter() {
		return null;
	}

} //VariableAdapterFactory
