 /**
 ********************************************************************************
 * Copyright (c) 2017 - 2018 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */
package org.eclipse.app4mc.emfutils.content.extractor.handlers;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.app4mc.emfutils.content.extractor.builders.EClassAllReferencesBufferBuilder;
import org.eclipse.app4mc.emfutils.content.extractor.utils.ExecutionCategory;
import org.eclipse.app4mc.emfutils.content.extractor.utils.ExecutionUtil;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;


public class EClassAllReferencesHandler extends AbstractHandlerUtils {

	private final EMap<String, Object> id2ObjectsMap;

	/**
	 * The constructor.
	 */
	public EClassAllReferencesHandler() {

		this.id2ObjectsMap = new BasicEMap<String, Object>();
	}

	/**
	 * the command has been executed, so extract extract the needed information from the application context.
	 */
	@Override
	public Object execute(final ExecutionEvent event) throws ExecutionException {


		StringBuffer plantumlBuffer = new StringBuffer();

		final EClassAllReferencesBufferBuilder builder = new EClassAllReferencesBufferBuilder();


		final String eClassName = "";


		ISelection selection = null;

		selection = HandlerUtil.getActiveMenuSelection(event);

		if (selection == null) {

			selection = HandlerUtil.getActiveWorkbenchWindow(event).getActivePage().getSelection();

		}
		if (selection != null & selection instanceof IStructuredSelection) {

			final IStructuredSelection strucSelection = (IStructuredSelection) selection;

			for (final Iterator<Object> iterator = strucSelection.iterator(); iterator.hasNext();) {

				final Object element = iterator.next();

				if (element instanceof EObject) {

					EClass eClass = null;

					if (element instanceof EClass) {
						eClass = (EClass) element;
					}
					else {
						eClass = ((EObject) element).eClass();
					}

					plantumlBuffer = builder.buildBuffer(eClass);

					this.id2ObjectsMap.addAll(builder.getId2ObjectsMap());
				}
				else {
					openErrorDialog();
					return null;
				}
			}
		}
		else {
			openErrorDialog();
			return null;
		}

		System.out.println(plantumlBuffer.toString());

		try {
			final Map<String, String> propsMap = new HashMap<String, String>();

			// propsMap.put(ExecutionCategory.isClassReference_Hierarchy_subclass_info_generation.toString(), "true");

			ExecutionUtil.setExecutionCategory(ExecutionCategory.eClassAllReferences);

			generateSVGAndDisplayDiagram(plantumlBuffer, eClassName + "_SuperTypes_References", this.id2ObjectsMap,
					propsMap);
		}
		catch (final IOException e) {
			e.printStackTrace();
		}

		return null;
	}


}
