/**
 * *******************************************************************************
 * Copyright (c) 2019 Robert Bosch GmbH and others.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 * *******************************************************************************
 */

package templates.m2m.constraints

import com.google.inject.Inject
import com.google.inject.Singleton
import com.inchron.realtime.root.model.DataFlowEdge
import com.inchron.realtime.root.model.VariableAccess
import java.util.List
import org.eclipse.app4mc.amalthea.model.AbstractEventChain
import org.eclipse.app4mc.amalthea.model.ChannelAccess
import org.eclipse.app4mc.amalthea.model.ChannelEvent
import org.eclipse.app4mc.amalthea.model.EventChainReference
import templates.AbstractAmaltheaInchronTransformer
import templates.m2m.sw.runnableItem.ChannelAccessTransformer
import templates.m2m.utils.DataFlowUtils

@Singleton
class EventChainReferenceTransformer extends AbstractAmaltheaInchronTransformer {
	
	@Inject EventChainTransformer eventChainTransformer
	@Inject ChannelAccessTransformer channelAccessTransformer

	
	def create inchronModelFactory.createDataFlowReference createDataFlowReference(EventChainReference amltEventEventChainReference, 
		AbstractEventChain amltAbstractEventChain) {
		
		it.dataFlow = eventChainTransformer.createDataFlow(amltEventEventChainReference.eventChain)

		// assume entire event chain is referenced		
		it.stimulus = getInitialStimulus(it.dataFlow.edges).stimulus
		it.response = getUltimateResponse(it.dataFlow.edges).response
		
			 	
	 	//check if a only a portion of the data floW is selected in parent Amalthea event chain
	 	val ChannelAccess amltCutoffStimulus = DataFlowUtils.findChannelAccess(amltAbstractEventChain.stimulus)
		val ChannelAccess amltCutoffResponse = DataFlowUtils.findChannelAccess(amltAbstractEventChain.response)
		
		val VariableAccess csCutoffStimulus = channelAccessTransformer.createVariableAccess((amltAbstractEventChain.stimulus as ChannelEvent).process, amltCutoffStimulus)
		val VariableAccess csCutoffResponse = channelAccessTransformer.createVariableAccess((amltAbstractEventChain.response as ChannelEvent).process, amltCutoffResponse)
		
		for(DataFlowEdge dataFlowEdge : dataFlow.edges){
			if (dataFlowEdge.stimulus === csCutoffStimulus){
				it.stimulus = csCutoffStimulus
			}
			if (dataFlowEdge.response === csCutoffResponse){
				it.response = csCutoffResponse
			}
		}

	}
	
		
	private def DataFlowEdge getInitialStimulus(List<DataFlowEdge> list) {
		for(DataFlowEdge item : list){
			if (isInitial(item, list)){
				return item
			}
		}
		return null	
	}
	
	private def Boolean isInitial(DataFlowEdge item, List<DataFlowEdge> list){
		for (DataFlowEdge listItem : list){
			if(item.stimulus === listItem.response){
				//check item's stimulus is another item's response --> checked item has predecessor --> not start of event chain
				return false
			}
		}
		return true
	}
	
		private def DataFlowEdge getUltimateResponse(List<DataFlowEdge> list) {
		for(DataFlowEdge item : list){
			if (isUltimate(item, list)){
				return item
			}
		}
		return null	
	}
	
	private def Boolean isUltimate(DataFlowEdge item, List<DataFlowEdge> list){
		for (DataFlowEdge listItem : list){
			if(item.response === listItem.stimulus){
				// item's response is list item's stimulus --> checked item has successor --> not end of event chain
				return false
			}
		}
		return true
	}

}
	
	

