/**
 * *******************************************************************************
 * Copyright (c) 2019 Robert Bosch GmbH and others.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 * *******************************************************************************
 */

package templates.m2m.hw

import java.math.BigInteger
import org.eclipse.app4mc.amalthea.model.AmaltheaServices
import org.eclipse.app4mc.amalthea.model.Memory
import templates.AbstractAmaltheaInchronTransformer
import com.google.inject.Inject
import com.google.inject.Singleton

@Singleton
class MemoryTransformer extends AbstractAmaltheaInchronTransformer {

	@Inject FrequencyDomainTransformer frequencyDomainTransformer
	
	def create inchronMemoryFactory.createMemory createMemory(Memory amltMemory) {

		it.name = amltMemory.name

		it.clock = frequencyDomainTransformer.createClock(amltMemory?.frequencyDomain)

		var amltDataSize = amltMemory?.definition.size

		if (amltDataSize !== null) {
			var bytes = AmaltheaServices.convertToBit(amltDataSize).divide(new BigInteger("8"))

			it.size = bytes.intValue

		}

		it.prescaler = 1d

	}
	
}
