/**
 * *******************************************************************************
 * Copyright (c) 2019 Robert Bosch GmbH and others.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 * *******************************************************************************
 */

package templates.m2m.utils

import java.util.LinkedList
import java.util.List
import org.eclipse.app4mc.amalthea.model.AbstractEventChain
import org.eclipse.app4mc.amalthea.model.Channel
import org.eclipse.app4mc.amalthea.model.ChannelAccess
import org.eclipse.app4mc.amalthea.model.ChannelEvent
import org.eclipse.app4mc.amalthea.model.ChannelEventType
import org.eclipse.app4mc.amalthea.model.ChannelReceive
import org.eclipse.app4mc.amalthea.model.ChannelSend
import org.eclipse.app4mc.amalthea.model.Event
import org.eclipse.app4mc.amalthea.model.EventChain
import org.eclipse.app4mc.amalthea.model.EventChainContainer
import org.eclipse.app4mc.amalthea.model.EventChainItem
import org.eclipse.app4mc.amalthea.model.EventChainReference
import org.eclipse.app4mc.amalthea.model.Runnable
import org.eclipse.app4mc.amalthea.model.RunnableItem
import org.eclipse.app4mc.amalthea.model.Process

class DataFlowUtils {
	
	static var List<AbstractEventChain> referencingEventChains
	
	public static def boolean isDataFlow(EventChain eventChain){
		isDataFlow(eventChain, false)
	}
	
	public static def boolean isDataFlow(EventChain eventChain, boolean isReference){
		if (!isReference){
			referencingEventChains = new LinkedList<AbstractEventChain>();
		}
		return evaluateEventChain(eventChain)
	}
		
	private static def boolean evaluateEventChain(AbstractEventChain eventChain){
		if (!eventChain.segments.isEmpty){
			for (EventChainItem eventChainItem : eventChain.segments){
				if (eventChainItem instanceof EventChainReference){
					referencingEventChains.add(eventChain)
					evaluateEventChainItem(eventChainItem as EventChainReference)
				} else if (eventChainItem instanceof EventChainContainer){
					evaluateEventChainItem(eventChainItem as EventChainContainer)
				} else {
					return false;
				}
			}
		} else {
			if (findChannelAccess(eventChain.stimulus) === null){
				return false
			}
			if (findChannelAccess(eventChain.response) === null){
				return false
			}
		}

		return true
	}
	
	
	private static def boolean evaluateEventChainItem(EventChainReference eventChainReference){
		if(referencingEventChains.contains(eventChainReference.eventChain)){
			//circular dependency within event chain references, as this instance is listed as referencing event chain
			//this is not valid --> endless loop and stack overflow
			return false
		}
		return isDataFlow(eventChainReference.eventChain, true)
	}
	
	private static def boolean evaluateEventChainItem(EventChainContainer eventChainContainer){
		return evaluateEventChain(eventChainContainer?.eventChain)
	}

	public static def dispatch ChannelAccess findChannelAccess(Event amltEvent) {
		//some event not of specialization type ChannelEvent
		return null
	}
	

 	public static def dispatch ChannelAccess findChannelAccess(ChannelEvent amltChannelEvent) {
		val Channel channel = amltChannelEvent.entity;
		val ChannelEventType eventType =  amltChannelEvent.eventType;
		val Runnable runnable = amltChannelEvent.runnable
		val Process task = amltChannelEvent.process
		
		if(runnable === null) 
			return null
		if(task === null)
			return null
			
		if(!runnable.taskRunnableCalls.map[it.containingProcess].contains(task))
			return null
		
		if (eventType == ChannelEventType.SEND) {
			return findChannelSend(runnable, channel)
		} else if (eventType == ChannelEventType.RECEIVE) {
			return findChannelReceive(runnable, channel)
		} else {
			return null;			
		}
	}
	
	private static def ChannelSend findChannelSend(Runnable runnable, Channel channel) {
		var ChannelSend ret = null
		for (RunnableItem runnableItem : runnable.runnableItems) {	
			if (runnableItem instanceof ChannelSend){
				val channelSend = runnableItem as ChannelSend
				if (channelSend.data === channel){
					if (ret === null){
						ret = channelSend
					} else{ 
						//multiple send accesses to this channel within runnable --> not valid
						return null
					}
				}
			}		
		}
		return ret
	}
	
	private static def ChannelReceive findChannelReceive(Runnable runnable, Channel channel) {	
		var ChannelReceive ret = null	
		for (RunnableItem runnableItem : runnable.runnableItems) {	
			if (runnableItem instanceof ChannelReceive){
				val channelReceive= runnableItem as ChannelReceive
				if (channelReceive.data === channel){
					if (ret === null){ 
						ret = channelReceive
					} else{ 
						//multiple receive accesses to this channel within runnable --> not valid
						return null
					}
				}
			}		
		}
		return ret
	}
	

}
