/**
 ********************************************************************************
 * Copyright (c) 2017 Robert Bosch GmbH and others.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */
package org.eclipse.app4mc.sca.ui.preferences.interfaces;

import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;

/**
 * The contribution class of the extension point org.eclipse.app4mc.ui.exportOptions need to implement the below
 * interface. The extension point expect the keys that are to be exported as part of the options file and the preference
 * store from which the value of the keys are to be fetched.
 */
public interface IExportPreferences {

  /**
   * Mentions all the preferences to be exported to the options file
   *
   * @return List of preferences to be exported
   */
  public List<String> keysToExport();

  /**
   * Specifies the preference store from which the preferences are to be exported
   *
   * @return {@link IPreferenceStore}
   */
  public IPreferenceStore getPreferenceStore();
}
