/* ========================================================================
 * PlantUML : a free UML diagram generator
 * ========================================================================
 *
 * (C) Copyright 2009-2014, Arnaud Roques
 *
 * Project Info:  http://plantuml.sourceforge.net
 * 
 * This file is part of PlantUML.
 *
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE PUBLIC
 * LICENSE ("AGREEMENT"). [Eclipse Public License - v 1.0]
 * 
 * ANY USE, REPRODUCTION OR DISTRIBUTION OF THE PROGRAM CONSTITUTES
 * RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 * 
 * You may obtain a copy of the License at
 * 
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *
 * Original Author:  Arnaud Roques
 */
package net.sourceforge.plantuml;

import net.sourceforge.plantuml.api.HealthCheck;
import net.sourceforge.plantuml.api.Performance;

public abstract class Log {

	private static final long start = System.currentTimeMillis();

	public synchronized static void debug(String s) {
	}

	public synchronized static void info(String s) {
		if (OptionFlags.getInstance().isVerbose()) {
			System.out.println(format(s));
		}
	}

	public synchronized static void error(String s) {
		System.err.println(s);
	}

	private static String format(String s) {
		final long delta = System.currentTimeMillis() - start;
		final HealthCheck healthCheck = Performance.getHealthCheck();
		final long cpu = healthCheck.jvmCpuTime() / 1000L / 1000L;
		final long dot = healthCheck.dotTime().getSum();
		final StringBuilder sb = new StringBuilder();
		sb.append("(");
		sb.append(delta / 1000L);
		sb.append(".");
		sb.append(String.format("%03d", delta % 1000L));
		if (cpu != -1) {
			sb.append(" - ");
			sb.append(cpu / 1000L);
			sb.append(".");
			sb.append(String.format("%03d", cpu % 1000L));
		}
		sb.append(" - ");
		sb.append(dot / 1000L);
		sb.append(".");
		sb.append(String.format("%03d", dot % 1000L));
		sb.append("(");
		sb.append(healthCheck.dotTime().getNb());
		sb.append(")");
		sb.append(" - ");
		final long total = (healthCheck.totalMemory()) / 1024 / 1024;
		final long free = (healthCheck.freeMemory()) / 1024 / 1024;
		sb.append(total);
		sb.append(" Mo) ");
		sb.append(free);
		sb.append(" Mo - ");
		sb.append(s);
		return sb.toString();

	}

	public static void println(Object s) {
		if (header == null) {
			System.err.println("L = " + s);
		} else {
			System.err.println(header + " " + s);
		}
	}

	private static String header;

	public static void header(String s) {
		header = s;
	}
}
